import { App } from 'vue';
import { Ref } from 'vue';

/**
 * Creates a in-memory based history. The main purpose of this history is to handle SSR. It starts in a special location that is nowhere.
 * It's up to the user to replace that location with the starter location.
 * @param base - Base applied to all urls, defaults to '/'
 * @returns a history object that can be passed to the router constructor
 */
export declare function createMemoryHistory(base?: string): RouterHistory;

export declare function createRouter({ history, routes, scrollBehavior, }: RouterOptions): Router;

export declare function createWebHashHistory(base?: string): RouterHistory;

export declare function createWebHistory(base?: string): RouterHistory;

declare type ErrorHandler = (error: any) => any;

declare interface HistoryLocation {
    fullPath: string;
    state?: HistoryState;
}

declare type HistoryLocationNormalized = Pick<HistoryLocation, 'fullPath'>;

declare interface HistoryState {
    [x: number]: HistoryStateValue;
    [x: string]: HistoryStateValue;
}

declare interface HistoryStateArray extends Array<HistoryStateValue> {
}

declare type HistoryStateValue = string | number | boolean | null | HistoryState | HistoryStateArray;

declare type Immutable<T> = {
    readonly [P in keyof T]: Immutable<T[P]>;
};

export declare const Link: new () => import("vue").ComponentPublicInstance<{
    to: RouteLocation;
} & {}, () => import("vue").VNode<any, any>, unknown, {}, {}, import("vue").VNodeProps & {
    to: RouteLocation;
} & {}>;

declare type ListenerRemover = () => void;

declare interface LocationAsName {
    name: string;
    params?: RouteParamsRaw;
}

declare interface LocationAsPath {
    path: string;
}

declare interface LocationAsRelative {
    params?: RouteParams;
}

declare type LocationQuery = Record<string, LocationQueryValue | LocationQueryValue[]>;

declare type LocationQueryRaw = Record<string | number, LocationQueryValueRaw | LocationQueryValueRaw[]>;

declare type LocationQueryValue = string | null;

declare type LocationQueryValueRaw = LocationQueryValue | number | undefined;

declare interface NavigationCallback {
    (to: HistoryLocationNormalized, from: HistoryLocationNormalized, information: NavigationInformation): void;
}

declare enum NavigationDirection {
    back = "back",
    forward = "forward",
    unknown = ""
}

declare interface NavigationGuard<V = void> {
    (this: V, to: Immutable<RouteLocationNormalized>, from: Immutable<RouteLocationNormalized>, next: NavigationGuardCallback): any;
}

declare interface NavigationGuardCallback {
    (): void;
    (location: RouteLocation): void;
    (valid: false): void;
    (cb: (vm: any) => void): void;
}

declare interface NavigationInformation {
    type: NavigationType;
    direction: NavigationDirection;
    distance: number;
}

declare enum NavigationType {
    pop = "pop",
    push = "push"
}

export declare function onBeforeRouteLeave(leaveGuard: NavigationGuard): void;

declare interface PathParserOptions {
    /**
     * Makes the RegExp case sensitive. Defaults to false
     */
    sensitive?: boolean;
    /**
     * Should we allow a trailing slash. Defaults to true
     */
    strict?: boolean;
    /**
     * Should the RegExp match from the beginning by prepending a ^. Defaults to true
     */
    start?: boolean;
    /**
     * Should the RegExp match until the end by appending a $. Defaults to true
     */
    end?: boolean;
}

declare interface PostNavigationGuard {
    (to: Immutable<RouteLocationNormalized>, from: Immutable<RouteLocationNormalized>): any;
}

declare type RawHistoryLocation = HistoryLocation | string;

declare type RouteComponent = TODO;

declare type RouteLocation = string | (RouteQueryAndHash & LocationAsPath & RouteLocationOptions) | (RouteQueryAndHash & LocationAsName & RouteLocationOptions) | (RouteQueryAndHash & LocationAsRelative & RouteLocationOptions);

export declare interface RouteLocationNormalized {
    path: string;
    fullPath: string;
    query: LocationQuery;
    hash: string;
    name: string | null | undefined;
    params: RouteParams;
    matched: RouteRecordNormalized[];
    redirectedFrom: RouteLocationNormalized | undefined;
    meta: Record<string | number | symbol, any>;
}

export declare interface RouteLocationOptions {
    /**
     * Replace the entry in the history instead of pushing a new entry
     */
    replace?: boolean;
    /**
     * Triggers the navigation even if the location is the same as the current one
     */
    force?: boolean;
}

declare type RouteParams = Record<string, RouteParamValue | RouteParamValue[]>;

declare type RouteParamsRaw = RouteParams;

declare type RouteParamValue = string;

declare interface RouteQueryAndHash {
    query?: LocationQueryRaw;
    hash?: string;
}

export declare interface Router {
    history: RouterHistory;
    currentRoute: Ref<Immutable<RouteLocationNormalized>>;
    addRoute(parentName: string, route: RouteRecord): () => void;
    addRoute(route: RouteRecord): () => void;
    removeRoute(name: string): void;
    getRoutes(): RouteRecordNormalized[];
    resolve(to: RouteLocation): RouteLocationNormalized;
    createHref(to: RouteLocationNormalized): string;
    push(to: RouteLocation): Promise<RouteLocationNormalized>;
    replace(to: RouteLocation): Promise<RouteLocationNormalized>;
    beforeEach(guard: NavigationGuard): ListenerRemover;
    afterEach(guard: PostNavigationGuard): ListenerRemover;
    onError(handler: ErrorHandler): ListenerRemover;
    isReady(): Promise<void>;
    install(app: App): void;
}

declare type RouteRecord = RouteRecordSingleView | RouteRecordMultipleViews | RouteRecordRedirect;

declare interface RouteRecordCommon {
    path: string;
    alias?: string | string[];
    name?: string;
    beforeEnter?: NavigationGuard | NavigationGuard[];
    meta?: Record<string | number | symbol, any>;
    options?: PathParserOptions;
}

declare interface RouteRecordMultipleViews extends RouteRecordCommon {
    components: Record<string, RouteComponent>;
    children?: RouteRecord[];
}

declare interface RouteRecordNormalized {
    path: RouteRecordMultipleViews['path'];
    name: RouteRecordMultipleViews['name'];
    components: RouteRecordMultipleViews['components'];
    children: Exclude<RouteRecordMultipleViews['children'], void>;
    meta: Exclude<RouteRecordMultipleViews['meta'], void>;
    beforeEnter: RouteRecordMultipleViews['beforeEnter'];
    leaveGuards: NavigationGuard[];
    aliasOf: RouteRecordNormalized | undefined;
}

declare interface RouteRecordRedirect extends RouteRecordCommon {
    redirect: RouteRecordRedirectOption;
    beforeEnter?: never;
    component?: never;
    components?: never;
}

declare type RouteRecordRedirectOption = RouteLocation | ((to: Immutable<RouteLocationNormalized>) => RouteLocation);

declare interface RouteRecordSingleView extends RouteRecordCommon {
    component: RouteComponent;
    children?: RouteRecord[];
}

declare interface RouterHistory {
    readonly base: string;
    readonly location: HistoryLocationNormalized;
    push(to: RawHistoryLocation): void;
    replace(to: RawHistoryLocation): void;
    back(triggerListeners?: boolean): void;
    forward(triggerListeners?: boolean): void;
    go(distance: number, triggerListeners?: boolean): void;
    listen(callback: NavigationCallback): ListenerRemover;
    destroy(): void;
}

declare interface RouterOptions {
    history: RouterHistory;
    routes: RouteRecord[];
    scrollBehavior?: ScrollBehavior;
}

declare interface ScrollBehavior {
    (to: RouteLocationNormalized, from: RouteLocationNormalized, savedPosition: ScrollToPosition | null): ScrollPosition | Promise<ScrollPosition>;
}

declare type ScrollPosition = ScrollToPosition | ScrollToElement;

declare interface ScrollToElement {
    selector: string;
    offset?: ScrollToPosition;
}

declare type ScrollToPosition = {
    x: number;
    y: number;
};

export declare const START_LOCATION: RouteLocationNormalized;

declare type TODO = any;

export declare function useRoute(): Ref<RouteLocationNormalized>;

export declare function useRouter(): Router;

export declare const View: new () => import("vue").ComponentPublicInstance<{
    name: string;
} & {}, () => import("vue").VNode<any, any> | null, unknown, {}, {}, import("vue").VNodeProps & {} & {
    name?: string | undefined;
}>;

export { }
